/* TeamSCOPE - Software for Collaborative Project Environments.
   Copyright (C) 1999, 2000 Michigan State University.
   Written by Ben Pfaff <pfaffben@msu.edu>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To contact the TeamSCOPE team, send email to scope@cscw.msu.edu, or
   write to Charles Steinfield, Room 436 Communication Arts Building,
   Michigan State University, Department of Telecommunication, East
   Lansing MI 48824-1212. */

#if !main_h
#define main_h 1

#include <stddef.h>

/* Session states. */
enum
  {
    CLS_INIT,			/* No session established. */
    CLS_READY,			/* Session established, waiting for command. */
    CLS_CLOSING,		/* Session will close after output flushed. */
    CLS_CLOSED			/* Session closed. */
  };

/* Client flags. */
enum
  {
    FLG_POPPED = 001,		/* Chat window popped up. */
    FLG_OTHERS_POPPED = 002,	/* Thinks others have chat window popped up. */
  };

/* Input buffer size.
   This is the maximum line length, including CR, LF, and one pad byte. */
#define IN_BUF_SIZE 512

/* Defines a client connection's total state. */
struct session
  {
    int sock;			/* Socket. */
    int state;			/* Client state. */
    int flags;			/* Client flags. */

    char *workgroup;		/* Current workgroup. */
    char *username;		/* Current username. */
    char *hostinfo;		/* client info: [aaa.bbb.ccc.ddd:ppppp]. */
    char *cookie;		/* Session cookie. */

    /* Input buffer. */
    char in_buf[IN_BUF_SIZE];	/* Data received. */
    int in_head, in_tail;	/* Head, tail of circular queue. */
    int in_newline;		/* Position of newline in buf. */
    int in_discard;		/* 1=drop current line. */

    /* Output buffer. */
    struct buffer *out_head;	/* Head of output FIFO. */
    struct buffer *out_tail;	/* Head of output FIFO. */
    size_t out_sent;		/* Amount of current buffer already sent. */
  };

/* File descriptor sets. */
#include <arpa/inet.h>
extern fd_set read_fds;
extern fd_set write_fds;

/* Sessions in progress. */
extern struct session **sessions;
extern int n_sessions;
extern int m_sessions;

void log (const char *, ...);
void fail (const char *, ...) attribute ((noreturn));

#endif /* main.h */
