/* TeamSCOPE - Software for Collaborative Project Environments.
   Copyright (C) 1998-9, 2000 Free Software Foundation, Inc.
   Copyright (C) 1999, 2000 Michigan State University.
   Written by Ben Pfaff <pfaffben@msu.edu>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To contact the TeamSCOPE team, send email to scope@cscw.msu.edu, or
   write to Charles Steinfield, Room 436 Communication Arts Building,
   Michigan State University, Department of Telecommunication, East
   Lansing MI 48824-1212. */

#if !str_h
#define str_h 1

/* Headers and miscellaneous. */

#include <stdarg.h>

#if STDC_HEADERS
#include <string.h>
#else
#ifndef HAVE_STRCHR
#define strchr index
#define strrchr rindex
#endif
char *strchr (), *strrchr ();
#endif

#if !HAVE_STRTOK_R
char *strtok_r (char *, const char *, char **);
#endif

#if !HAVE_STPCPY && !__linux__
char *stpcpy (char *dest, const char *src);
#endif

#if !HAVE_STRCASECMP
int strcasecmp (const char *s1, const char *s2);
#endif

#if !HAVE_STRNCASECMP
int strncasecmp (const char *s1, const char *s2, size_t n);
#endif

#if !HAVE_MEMMEM
const void *memmem (const void *haystack, size_t haystack_len,
		    const void *needle, size_t needle_len);
#endif

#define streq(A, B) 				\
	(!strcmp ((A), (B)))

/* spprintf() calls sprintf() and returns the address of the null
   terminator in the resulting string.  It should be portable the way
   it's been implemented. */
#if __GNUC__
#if HAVE_GOOD_SPRINTF
#define spprintf(buf, format, args...)			\
	((buf) + sprintf((buf), (format) , ## args))
#else /* not good sprintf */
#define spprintf(buf, format, args...)					\
	({ sprintf((buf), (format) , ## args); strchr((buf), 0); })
#endif /* not good sprintf */
#else /* not gcc */
char *spprintf (char *buf, const char *format,...);
#endif /* not gcc */

/* nsprintf() calls sprintf() and returns the strlen() of the
   resulting string.  It should be portable the way it's been
   implemented. */
#if __GNUC__
#if HAVE_GOOD_SPRINTF
#define nsprintf(BUF, FORMAT, ARGS...) 		\
	(sprintf ((BUF), (FORMAT) , ## ARGS))
#define nvsprintf(BUF, FORMAT, ARGS) 		\
	(vsprintf ((BUF), (FORMAT), (ARGS)))
#else /* not good sprintf */
#define nsprintf(BUF, FORMAT, ARGS...)		\
	({					\
	  char *_buf = BUF;			\
	  sprintf ((_buf), (FORMAT) , ## ARGS);	\
	  strlen (_buf);			\
	})
#define nvsprintf(BUF, FORMAT, ARGS)		\
	({					\
	  char *_buf = BUF;			\
	  vsprintf ((_buf), (FORMAT), (ARGS));	\
	  strlen (_buf);			\
	})
#endif /* not good sprintf */
#else /* not gcc */
#if HAVE_GOOD_SPRINTF
#define nsprintf sprintf
#define nvsprintf vsprintf
#else /* not good sprintf */
int nsprintf (char *buf, const char *format,...);
int nvsprintf (char *buf, const char *format, va_list args);
#endif /* not good sprintf */
#endif /* not gcc */

/* Custom string functions. */

#define stol(STRING) (strtol (STRING, NULL, 10))
char *blp_strcasestr (const char *haystack, const char *needle);
void blp_strip_trailing_spaces (char *s);
int has_prefix (const char *string, const char *prefix);

char *blp_asprintf (const char *fmt, ...)
     attribute ((format (printf, 1, 2)));

const char *format (const char *fmt, ...)
     attribute ((format (printf, 1, 2)));

size_t vformat (char *buffer, size_t size, const char *format, va_list args);

/* Dynamic strings. */

struct string
  {
    size_t len;
    size_t size;
    char *s;
  };

void st_create (struct string *, const char *);
void st_init (struct string *, size_t size);
void st_destroy (struct string *);
void st_clear (struct string *);
void st_extend (struct string *, size_t min_size);
void st_truncate (struct string *, size_t length);

size_t st_length (const struct string *);
char *st_value (const struct string *);

struct tm;

void st_putchar (struct string *, int ch);
void st_concat (struct string *, const char *);
void st_concat_quoted (struct string *, const char *, int quote);
void st_concat_url_query_quoted (struct string *, const char *);
void st_concat_html_quoted (struct string *st, const char *s);
void st_concat_buffer (struct string *st, const char *buf, size_t len);
void st_printf (struct string *, const char *, ...)
     attribute ((format (printf, 2, 3)));
void st_strftime (struct string *st, const char *format,
		  const struct tm *date);

#endif /* str_h */
