/* TeamSCOPE - Software for Collaborative Project Environments.
   Copyright (C) 1999, 2000, 2001 Michigan State University.
   Written by Ben Pfaff <pfaffben@msu.edu>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To contact the TeamSCOPE team, send email to scope@cscw.msu.edu, or
   write to Charles Steinfield, Room 436 Communication Arts Building,
   Michigan State University, Department of Telecommunication, East
   Lansing MI 48824-1212. */

#include <config.h>
#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include "scope.cgi.h"
#include "str.h"

/* Public functions. */

extern void msg_prefix (FILE *file, const char *function_name);

void
open_log_file (const char *filename)
{
  assert (filename != NULL);
  if (freopen (filename, "a", stderr) == NULL)
    abort ();

  setvbuf (stderr, NULL, _IOLBF, BUFSIZ);
}
    
#if __GNUC__ >= 2
/* Writes printf()-style message FORMAT to stderr. */
void 
(log) (const char *function_name, const char *format, ...)
{
  va_list args;

  msg_prefix (stderr, function_name);
  va_start (args, format);
  vfprintf (stderr, format, args);
  va_end (args);

  putc ('\n', stderr);
}
#else /* !__GNUC__ */
/* Writes printf()-style message FORMAT to stderr. */
void 
log (const char *format, ...)
{
  va_list args;

  msg_prefix (stderr, NULL);
  va_start (args, format);
  vfprintf (stderr, format, args);
  va_end (args);

  putc ('\n', stderr);
}
#endif /* !__GNUC__ */

/* Writes printf()-style message FORMAT to stderr, preceded by
   `fatal:', then aborts program execution unsuccessfully. */
void 
fail (const char *format, ...)
{
  va_list args;

  msg_prefix (stderr, NULL);
  fputs ("fatal: ", stderr);
  va_start (args, format);
  vfprintf (stderr, format, args);
  va_end (args);
  
  putc ('\n', stderr);

  exit (EXIT_FAILURE);
}
