/* TeamSCOPE - Software for Collaborative Project Environments.
   Copyright (C) 1999, 2000, 2001 Michigan State University.
   Written by Ben Pfaff <pfaffben@msu.edu>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To contact the TeamSCOPE team, send email to scope@cscw.msu.edu, or
   write to Charles Steinfield, Room 436 Communication Arts Building,
   Michigan State University, Department of Telecommunication, East
   Lansing MI 48824-1212. */

#if !html_h
#define html_h 1

#if GLOBAL_DEBUGGING
#include <time.h>
#include <sys/time.h>

extern clock_t start_clock;
extern struct timeval start_timeval;
#endif

extern const char *server_protocol;	/* http */
extern const char *server_name;	/* cscw.msu.edu */
extern int server_port;		/* 80 */
extern const char *www_root;	/* /scope */
extern const char *script_name;	/* /scope/scope.cgi */
extern const char *mail_domain;	/* cscw.msu.edu */
extern const char *admin_email;	/* scope@cscw.msu.edu */
extern const char *our_abs_url;	/* http://cscw.msu.edu/scope/scope.cgi */

void html_initialize (void);

struct request;
struct context;
struct std;
struct url;

void http_header (void);
void html_header (void);
void html_powered_by (struct request *);

struct st *html_buffer;
void html_push_buffer (struct st *);
void html_pop_buffer (void);
void html (const char *, ...) attribute ((format (printf, 1, 2)));
void htmls (const char *);
void htmlc (int);
void html_flush (void);

void html_escape (const char *);
void html_pretty_print (const char *);
void html_list (int level);
void html_a_file (struct context *, const char *filename, const struct std *);
void html_button (const char *screen, const char *label, ...);

void html_banner (struct request *);
void html_trailer (struct request *);
void html_hidden_abbrev (struct request *);
void html_chat_applet (struct request *);

/* Form methods. */
enum
  {
    HT_GET=1,		/* GET. */
    HT_POST=2		/* POST. */
  };

/* Form types.  */
enum
  {
    HT_URL_ENCODED=3,	/* application/x-www-url-encoded. */
    HT_FORM_DATA=4	/* multipart/form-data. */
  };

void html_start_form (const char *screen, int method, int type);
void html_end_form (void);
void html_text_field (const char *name, const char *def, int size);
void html_password_field (const char *name, int size);
void html_file_field (const char *name);
void html_radio (const char *name, const char *value, const char *label,
		 int checked);
void html_checkbox (const char *name, int checked, const char *label);
void html_text_area (const char *name, const char *def, int rows, int columns);
void html_hidden (const char *name, const char *value);
void html_submit (const char *label);
void html_submit_alt (const char *name, const char *label);
void html_label (const char *name, const char *label);

#include "str.h"

struct url
  {
    struct st s;
    int n_args;
    int mark_length;
    int mark_n_args;
  };

void url_create (struct url *, const char *screen);
void url_copy (struct url *dst, const struct url *src);
void url_destroy (struct url *);
void url_arg (struct url *, const char *, const char *);
void url_arg_int (struct url *, const char *, int);
void url_arg_std (struct url *, const struct std *);
void url_copy_abbrev (struct url *, struct request *);
void url_mark (struct url *);
void url_release (struct url *);
const char *url_string (const struct url *);
const char *url_this (const char *, ...);
const char *url_file (const char *, const struct std *);

#endif /* html.h */
