/* TeamSCOPE - Software for Collaborative Project Environments.
   Copyright (C) 2000, 2001 Michigan State University.
   Written by Ben Pfaff <pfaffben@msu.edu>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To contact the TeamSCOPE team, send email to scope@cscw.msu.edu, or
   write to Charles Steinfield, Room 436 Communication Arts Building,
   Michigan State University, Department of Telecommunication, East
   Lansing MI 48824-1212. */

#ifndef id_list_h
#define id_list_h 1

/* ADT for a list of IDs. */

#include "db.h"

/* List of IDs. */
struct id_list
  {
    size_t n;	/* Number of IDs. */
    char **s;	/* IDs, dynamically allocated, in sorted order. */
  };

struct st;
struct std;
void id_list_null (struct id_list *);
int id_list_null_p (const struct id_list *);
void id_list_destroy (struct id_list *);
void id_list_copy (struct id_list *, const struct id_list *);
void id_list_one (struct id_list *, const char *);
void id_list_all (struct id_list *);
int id_list_equal (const struct id_list *, const struct id_list *);
void id_list_from_query (struct id_list *list, db_result *r);
void id_list_from_std (struct id_list *, const struct std *, const char *,
		       size_t max);
void id_list_to_std (struct id_list *, struct std *, const char *);
void id_list_to_sql (struct id_list *, struct st *);
void id_list_for_group (struct id_list *, const char *);
void id_list_for_user (struct id_list *list, const char *usr);
void id_list_one_group (struct id_list *);
void id_list_intersect (struct id_list *, struct id_list *);
void id_list_subtract (struct id_list *la, struct id_list *lb);
void id_list_sort (struct id_list *);
int id_list_find (const struct id_list *, const char *id);
int id_list_contains (struct id_list *list, const char *id);
void id_list_print (const struct id_list *);

#endif /* id-list.h */
