/* TeamSCOPE - Software for Collaborative Project Environments.
   Copyright (C) 1999, 2000, 2001 Michigan State University.
   Written by Ben Pfaff <pfaffben@msu.edu>.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To contact the TeamSCOPE team, send email to scope@cscw.msu.edu, or
   write to Charles Steinfield, Room 436 Communication Arts Building,
   Michigan State University, Department of Telecommunication, East
   Lansing MI 48824-1212. */

#if !std_h
#define std_h 1

#include <sys/types.h>
#include "avl.h"

/* An iterator. */
struct std_iterator
  {
    struct avl_tree *tree;
    struct avl_traverser trav;
    struct st_entry *entry;
    int index;
  };

/* Flags to std_read. */
enum
  {
    STDF_SUBST = 001,		/* Make variable substitutions. */
    STDF_TRIM_SPACES = 002,	/* Trim leading and trailing spaces. */

    STDF_LIBERAL = 004,		/* Internal use only. */
  };

/* Public functions. */
struct std *std_create (int allow_dup);
void std_destroy (struct std *);

int std_count (struct std *);

int std_put (struct std *, const char *key, const char *value);
const char *std_get (const struct std *, const char *key);

void std_replace (struct std *, const char *key, const char *value);
void std_delete (struct std *, const char *key);

int std_contains (const struct std *, const char *key);
int std_value_contains (const struct std *,
			const char *key, const char *value);
const char *std_get_null (const struct std *, const char *key);
const char *std_get_nonempty (const struct std *, const char *key);
const char *std_get_assert (const struct std *, const char *key);
const char **std_get_many (const struct std *, const char *key, int *n);

struct std *std_get_subdict (struct std *, const char *key);
void std_put_subdict (struct std *, const char *key, const struct std *value);

void std_init_iterator (const struct std *dict, struct std_iterator *iter);
int std_iterate (struct std_iterator *iter, const char **key, char **value);

int std_read (struct std *dict, const char *filename, int flags, uid_t uid);
int std_write (const struct std *dict, const char *filename, mode_t);
int std_modified (const struct std *dict);

#if GLOBAL_DEBUGGING
void std_print (const struct std *dict);
#endif

int lock (int type, int fd);

#endif /* std_h */

